@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -264,27 +264,83 @@ static void network_ready(struct connman_device *device)
 								device, NULL);
 }
 
-static void creg_update(struct connman_device *device, int status)
+static gboolean lost_network(int old, int new)
+{
+	if (old != 1 && old != 5)
+		return FALSE;
+
+	if (new == 1 || new == 5)
+		return FALSE;
+
+	return TRUE;
+}
+
+static gboolean get_network(int old, int new)
+{
+	if (old == 1 || old == 5)
+		return FALSE;
+
+	if (new != 1 && new != 5)
+		return FALSE;
+
+	return TRUE;
+}
+
+static void cleanup_network(struct connman_device *device)
+{
+	struct mbm_data *data = connman_device_get_data(device);
+	const char *identifier;
+
+	DBG("");
+
+	if (data->network == NULL)
+		return;
+
+	connman_network_set_connected(data->network, FALSE);
+
+	identifier = connman_network_get_identifier(data->network);
+
+	connman_device_remove_network(device, identifier);
+
+	data->network = NULL;
+}
+
+static void update_roaming(struct connman_device *device, int status)
 {
 	struct mbm_data *data = connman_device_get_data(device);
 
+	if (data->network == NULL)
+		return;
+
 	if (status != 1 && status != 5)
-		goto done;
+		return;
 
-	if (data->network != NULL) {
-		if (status == 5)
-			connman_network_set_roaming(data->network, TRUE);
-		else
-			connman_network_set_roaming(data->network, FALSE);
+	if (status == 1)
+		connman_network_set_roaming(data->network, FALSE);
+	else
+		connman_network_set_roaming(data->network, TRUE);
 
-		connman_network_set_group(data->network, data->imsi);
+	connman_network_set_group(data->network, data->imsi);
+}
+
+static void creg_update(struct connman_device *device, int status)
+{
+	struct mbm_data *data = connman_device_get_data(device);
+	int old_status = data->creg_status;
+
+	DBG("old_status %d status %d", old_status, status);
+
+	data->creg_status = status;
+
+	if (lost_network(old_status, status) == TRUE) {
+		cleanup_network(device);
+		return;
 	}
 
-	if (data->creg_status != 1 && data->creg_status != 5)
+	if (get_network(old_status, status) == TRUE)
 		network_ready(device);
 
-done:
-	data->creg_status = status;
+	update_roaming(device, status);
 }
 
 static void creg_query(gboolean ok, GAtResult *result,
@@ -548,8 +604,14 @@ static int network_probe(struct connman_network *network)
 
 static void network_remove(struct connman_network *network)
 {
+	struct connman_device *device = connman_network_get_device(network);
+	struct mbm_data *data;
+
 	DBG("network %p", network);
 
+	data = connman_device_get_data(device);
+	data->network = NULL;
+
 	connman_network_set_data(network, NULL);
 }
 
@@ -608,6 +670,8 @@ static void mbm_newlink(unsigned flags, unsigned change, void *user_data)
 
 	if ((data->flags & IFF_LOWER_UP) != (flags & IFF_LOWER_UP)) {
 		if (flags & IFF_LOWER_UP) {
+			connman_network_set_method(data->network,
+					CONNMAN_IPCONFIG_METHOD_DHCP);
 			connman_network_set_connected(data->network, TRUE);
 		} else {
 			connman_network_set_connected(data->network, FALSE);
