@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -25,6 +25,7 @@
 
 #include <net/if.h>
 #include <net/if_arp.h>
+#include <linux/if_link.h>
 
 #ifndef IFF_LOWER_UP
 #define IFF_LOWER_UP	0x10000
@@ -46,9 +47,6 @@ struct connman_ipconfig {
 	enum connman_ipconfig_method method;
 	struct connman_ipaddress *address;
 	struct connman_ipaddress *system;
-
-	char *eth;
-	uint16_t mtu;
 };
 
 struct connman_ipdevice {
@@ -56,6 +54,10 @@ struct connman_ipdevice {
 	char *ifname;
 	unsigned short type;
 	unsigned int flags;
+	char *address;
+	uint16_t mtu;
+	uint32_t tx_bytes;
+	uint32_t rx_bytes;
 
 	GSList *address_list;
 	char *gateway;
@@ -231,6 +233,7 @@ static void free_ipdevice(gpointer data)
 	free_address_list(ipdevice);
 	g_free(ipdevice->gateway);
 
+	g_free(ipdevice->address);
 	g_free(ipdevice->ifname);
 	g_free(ipdevice);
 }
@@ -337,9 +340,31 @@ static void __connman_ipconfig_lower_down(struct connman_ipdevice *ipdevice)
 	connman_inet_clear_address(ipdevice->index);
 }
 
+static void update_stats(struct connman_ipdevice *ipdevice,
+						struct rtnl_link_stats *stats)
+{
+	if (stats->rx_packets == 0 && stats->tx_packets == 0)
+		return;
+
+	connman_info("%s {RX} %u packets %u bytes", ipdevice->ifname,
+					stats->rx_packets, stats->rx_bytes);
+	connman_info("%s {TX} %u packets %u bytes", ipdevice->ifname,
+					stats->tx_packets, stats->tx_bytes);
+
+	if (ipdevice->config == NULL)
+		return;
+
+	ipdevice->rx_bytes = stats->rx_bytes;
+	ipdevice->tx_bytes = stats->tx_bytes;
+
+	__connman_counter_notify(ipdevice->ifname,
+				ipdevice->rx_bytes, ipdevice->tx_bytes);
+}
+
 void __connman_ipconfig_newlink(int index, unsigned short type,
 				unsigned int flags, const char *address,
-							unsigned short mtu)
+							unsigned short mtu,
+						struct rtnl_link_stats *stats)
 {
 	struct connman_ipdevice *ipdevice;
 	GList *list;
@@ -375,17 +400,17 @@ void __connman_ipconfig_newlink(int index, unsigned short type,
 	ipdevice->ifname = ifname;
 	ipdevice->type = type;
 
+	ipdevice->address = g_strdup(address);
+
 	g_hash_table_insert(ipdevice_hash, GINT_TO_POINTER(index), ipdevice);
 
 	connman_info("%s {create} index %d type %d <%s>", ipdevice->ifname,
 						index, type, type2str(type));
 
 update:
-	if (ipdevice->config != NULL) {
-		g_free(ipdevice->config->eth);
-		ipdevice->config->eth = g_strdup(address);
-		ipdevice->config->mtu = mtu;
-	}
+	ipdevice->mtu = mtu;
+
+	update_stats(ipdevice, stats);
 
 	if (flags == ipdevice->flags)
 		return;
@@ -455,7 +480,7 @@ update:
 		__connman_ipconfig_lower_down(ipdevice);
 }
 
-void __connman_ipconfig_dellink(int index)
+void __connman_ipconfig_dellink(int index, struct rtnl_link_stats *stats)
 {
 	struct connman_ipdevice *ipdevice;
 	GList *list;
@@ -466,6 +491,8 @@ void __connman_ipconfig_dellink(int index)
 	if (ipdevice == NULL)
 		return;
 
+	update_stats(ipdevice, stats);
+
 	for (list = g_list_first(ipconfig_list); list;
 						list = g_list_next(list)) {
 		struct connman_ipconfig *ipconfig = list->data;
@@ -771,7 +798,6 @@ void connman_ipconfig_unref(struct connman_ipconfig *ipconfig)
 
 		connman_ipaddress_free(ipconfig->system);
 		connman_ipaddress_free(ipconfig->address);
-		g_free(ipconfig->eth);
 		g_free(ipconfig);
 	}
 }
@@ -862,6 +888,15 @@ int connman_ipconfig_set_method(struct connman_ipconfig *ipconfig,
 	return 0;
 }
 
+enum connman_ipconfig_method __connman_ipconfig_get_method(
+				struct connman_ipconfig *ipconfig)
+{
+	if (ipconfig == NULL)
+		return CONNMAN_IPCONFIG_METHOD_UNKNOWN;
+
+	return ipconfig->method;
+}
+
 /**
  * connman_ipconfig_bind:
  * @ipconfig: ipconfig structure
@@ -881,6 +916,42 @@ void connman_ipconfig_bind(struct connman_ipconfig *ipconfig,
 	connman_inet_set_address(origin->index, origin->address);
 }
 
+int __connman_ipconfig_set_address(struct connman_ipconfig *ipconfig)
+{
+	DBG("");
+
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return connman_inet_set_address(ipconfig->index,
+						ipconfig->address);
+	}
+
+	return 0;
+}
+
+int __connman_ipconfig_clear_address(struct connman_ipconfig *ipconfig)
+{
+	DBG("");
+
+	switch (ipconfig->method) {
+	case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
+	case CONNMAN_IPCONFIG_METHOD_OFF:
+	case CONNMAN_IPCONFIG_METHOD_FIXED:
+	case CONNMAN_IPCONFIG_METHOD_DHCP:
+		break;
+	case CONNMAN_IPCONFIG_METHOD_MANUAL:
+		return connman_inet_clear_address(ipconfig->index);
+	}
+
+	return 0;
+
+}
+
 int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 {
 	struct connman_ipdevice *ipdevice;
@@ -895,6 +966,9 @@ int __connman_ipconfig_enable(struct connman_ipconfig *ipconfig)
 	if (ipdevice == NULL)
 		return -ENXIO;
 
+	if (ipdevice->config == ipconfig)
+		return -EALREADY;
+
 	if (ipdevice->config != NULL) {
 		ipconfig_list = g_list_remove(ipconfig_list, ipconfig);
 
@@ -1118,38 +1192,115 @@ int __connman_ipconfig_set_ipv4config(struct connman_ipconfig *ipconfig,
 	return 0;
 }
 
-int __connman_ipconfig_append_ethernet(struct connman_ipconfig *ipconfig,
+void __connman_ipconfig_append_proxy(struct connman_ipconfig *ipconfig,
 							DBusMessageIter *iter)
 {
+	const char *method = "direct";
+
+	connman_dbus_dict_append_basic(iter, "Method",
+						DBUS_TYPE_STRING, &method);
+}
+
+void __connman_ipconfig_append_ethernet(struct connman_ipconfig *ipconfig,
+							DBusMessageIter *iter)
+{
+	struct connman_ipdevice *ipdevice;
 	const char *method = "auto";
 
 	connman_dbus_dict_append_basic(iter, "Method",
 						DBUS_TYPE_STRING, &method);
 
-	if (ipconfig->eth != NULL)
+	ipdevice = g_hash_table_lookup(ipdevice_hash,
+					GINT_TO_POINTER(ipconfig->index));
+	if (ipdevice == NULL)
+		return;
+
+	if (ipdevice->ifname != NULL)
+		connman_dbus_dict_append_basic(iter, "Interface",
+					DBUS_TYPE_STRING, &ipdevice->ifname);
+
+	if (ipdevice->address != NULL)
 		connman_dbus_dict_append_basic(iter, "Address",
-					DBUS_TYPE_STRING, &ipconfig->eth);
+					DBUS_TYPE_STRING, &ipdevice->address);
 
-	if (ipconfig->mtu > 0)
+	if (ipdevice->mtu > 0)
 		connman_dbus_dict_append_basic(iter, "MTU",
-					DBUS_TYPE_UINT16, &ipconfig->mtu);
-
-	return 0;
+					DBUS_TYPE_UINT16, &ipdevice->mtu);
 }
 
 int __connman_ipconfig_load(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix)
 {
+	const char *method;
+	char *key;
+
 	DBG("ipconfig %p identifier %s", ipconfig, identifier);
 
+	key = g_strdup_printf("%smethod", prefix);
+	method = g_key_file_get_string(keyfile, identifier, key, NULL);
+	ipconfig->method = __connman_ipconfig_string2method(method);
+	g_free(key);
+
+	key = g_strdup_printf("%snetmask_prefixlen", prefix);
+	ipconfig->address->prefixlen = g_key_file_get_integer(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%slocal_address", prefix);
+	ipconfig->address->local = g_key_file_get_string(
+			keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%speer_address", prefix);
+	ipconfig->address->peer = g_key_file_get_string(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
+	key = g_strdup_printf("%sbroadcast_address", prefix);
+	ipconfig->address->broadcast = g_key_file_get_string(
+				keyfile, identifier, key, NULL);
+	g_free(key);
+
 	return 0;
 }
 
 int __connman_ipconfig_save(struct connman_ipconfig *ipconfig,
 		GKeyFile *keyfile, const char *identifier, const char *prefix)
 {
+	const char *method;
+	char *key;
+
 	DBG("ipconfig %p identifier %s", ipconfig, identifier);
 
+	method = __connman_ipconfig_method2string(ipconfig->method);
+
+	key = g_strdup_printf("%smethod", prefix);
+	g_key_file_set_string(keyfile, identifier, key, method);
+	g_free(key);
+
+	key = g_strdup_printf("%snetmask_prefixlen", prefix);
+	g_key_file_set_integer(keyfile, identifier,
+			key, ipconfig->address->prefixlen);
+	g_free(key);
+
+	key = g_strdup_printf("%slocal_address", prefix);
+	if (ipconfig->address->local != NULL)
+		g_key_file_set_string(keyfile, identifier,
+				key, ipconfig->address->local);
+	g_free(key);
+
+	key = g_strdup_printf("%speer_address", prefix);
+	if (ipconfig->address->peer != NULL)
+		g_key_file_set_string(keyfile, identifier,
+				key, ipconfig->address->peer);
+	g_free(key);
+
+	key = g_strdup_printf("%sbroadcast_address", prefix);
+	if (ipconfig->address->broadcast != NULL)
+		g_key_file_set_string(keyfile, identifier,
+			"broadcast_address", ipconfig->address->broadcast);
+	g_free(key);
+
 	return 0;
 }
 
