@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -34,6 +34,10 @@
 
 #include <gdbus.h>
 
+#ifdef HAVE_CAPNG
+#include <cap-ng.h>
+#endif
+
 #include "connman.h"
 
 static GMainLoop *main_loop = NULL;
@@ -45,11 +49,6 @@ static void sig_term(int sig)
 	g_main_loop_quit(main_loop);
 }
 
-static void sig_debug(int sig)
-{
-	__connman_toggle_debug();
-}
-
 static void disconnect_callback(DBusConnection *conn, void *user_data)
 {
 	connman_error("D-Bus disconnect");
@@ -57,6 +56,7 @@ static void disconnect_callback(DBusConnection *conn, void *user_data)
 	g_main_loop_quit(main_loop);
 }
 
+static gchar *option_debug = NULL;
 static gchar *option_device = NULL;
 static gchar *option_plugin = NULL;
 static gchar *option_nodevice = NULL;
@@ -64,11 +64,12 @@ static gchar *option_noplugin = NULL;
 static gchar *option_wifi = NULL;
 static gboolean option_detach = TRUE;
 static gboolean option_compat = FALSE;
-static gboolean option_debug = FALSE;
 static gboolean option_selftest = FALSE;
 static gboolean option_version = FALSE;
 
 static GOptionEntry options[] = {
+	{ "debug", 'd', 0, G_OPTION_ARG_STRING, &option_debug,
+				"Specify debug options to enable", "DEBUG" },
 	{ "device", 'i', 0, G_OPTION_ARG_STRING, &option_device,
 			"Specify networking device or interface", "DEV" },
 	{ "nodevice", 'I', 0, G_OPTION_ARG_STRING, &option_nodevice,
@@ -84,8 +85,6 @@ static GOptionEntry options[] = {
 				"Don't fork daemon to background" },
 	{ "compat", 'c', 0, G_OPTION_ARG_NONE, &option_compat,
 				"Enable Network Manager compatibility" },
-	{ "debug", 'd', 0, G_OPTION_ARG_NONE, &option_debug,
-				"Enable debug information output" },
 	{ "selftest", 't', 0, G_OPTION_ARG_NONE, &option_selftest,
 				"Run self testing routines" },
 	{ "version", 'v', 0, G_OPTION_ARG_NONE, &option_version,
@@ -114,6 +113,10 @@ int main(int argc, char *argv[])
 	struct sigaction sa;
 	mode_t old_umask;
 
+#ifdef HAVE_CAPNG
+	/* Drop capabilities */
+#endif
+
 #ifdef NEED_THREADS
 	if (g_thread_supported() == FALSE)
 		g_thread_init(NULL);
@@ -189,7 +192,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	__connman_log_init(option_detach, option_debug);
+	__connman_log_init(option_debug, option_detach);
 
 	if (option_selftest == TRUE) {
 		if (__connman_selftest() < 0) {
@@ -204,8 +207,10 @@ int main(int argc, char *argv[])
 	__connman_element_init(option_device, option_nodevice);
 
 	__connman_agent_init();
+	__connman_counter_init();
 	__connman_manager_init(option_compat);
 	__connman_profile_init();
+	__connman_config_init();
 
 	__connman_resolver_init();
 	__connman_ipconfig_init();
@@ -227,9 +232,6 @@ int main(int argc, char *argv[])
 	sigaction(SIGINT, &sa, NULL);
 	sigaction(SIGTERM, &sa, NULL);
 
-	sa.sa_handler = sig_debug;
-	sigaction(SIGUSR2, &sa, NULL);
-
 	g_main_loop_run(main_loop);
 
 	__connman_element_stop();
@@ -242,8 +244,10 @@ int main(int argc, char *argv[])
 	__connman_ipconfig_cleanup();
 	__connman_resolver_cleanup();
 
+	__connman_config_cleanup();
 	__connman_profile_cleanup();
 	__connman_manager_cleanup();
+	__connman_counter_cleanup();
 	__connman_agent_cleanup();
 
 	__connman_element_cleanup();
