@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -70,38 +70,18 @@ void connman_notifier_unregister(struct connman_notifier *notifier)
 	notifier_list = g_slist_remove(notifier_list, notifier);
 }
 
-static const char *type2string(enum connman_service_type type)
-{
-	switch (type) {
-	case CONNMAN_SERVICE_TYPE_UNKNOWN:
-		break;
-	case CONNMAN_SERVICE_TYPE_ETHERNET:
-		return "ethernet";
-	case CONNMAN_SERVICE_TYPE_WIFI:
-		return "wifi";
-	case CONNMAN_SERVICE_TYPE_WIMAX:
-		return "wimax";
-	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
-		return "bluetooth";
-	case CONNMAN_SERVICE_TYPE_CELLULAR:
-		return "cellular";
-	}
-
-	return NULL;
-}
-
 #define MAX_TECHNOLOGIES 10
 
 static volatile gint registered[MAX_TECHNOLOGIES];
 static volatile gint enabled[MAX_TECHNOLOGIES];
 static volatile gint connected[MAX_TECHNOLOGIES];
 
-void __connman_notifier_list_registered(DBusMessageIter *iter)
+void __connman_notifier_list_registered(DBusMessageIter *iter, void *user_data)
 {
 	int i;
 
-	for (i = 0; i < 10; i++) {
-		const char *type = type2string(i);
+	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+		const char *type = __connman_service_type2string(i);
 
 		if (type == NULL)
 			continue;
@@ -112,12 +92,12 @@ void __connman_notifier_list_registered(DBusMessageIter *iter)
 	}
 }
 
-void __connman_notifier_list_enabled(DBusMessageIter *iter)
+void __connman_notifier_list_enabled(DBusMessageIter *iter, void *user_data)
 {
 	int i;
 
-	for (i = 0; i < 10; i++) {
-		const char *type = type2string(i);
+	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+		const char *type = __connman_service_type2string(i);
 
 		if (type == NULL)
 			continue;
@@ -128,12 +108,12 @@ void __connman_notifier_list_enabled(DBusMessageIter *iter)
 	}
 }
 
-void __connman_notifier_list_connected(DBusMessageIter *iter)
+void __connman_notifier_list_connected(DBusMessageIter *iter, void *user_data)
 {
 	int i;
 
-	for (i = 0; i < 10; i++) {
-		const char *type = type2string(i);
+	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+		const char *type = __connman_service_type2string(i);
 
 		if (type == NULL)
 			continue;
@@ -147,106 +127,91 @@ void __connman_notifier_list_connected(DBusMessageIter *iter)
 static void technology_registered(enum connman_service_type type,
 						connman_bool_t registered)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value, iter;
-	const char *key = "AvailableTechnologies";
-
 	DBG("type %d registered %d", type, registered);
 
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-					DBUS_TYPE_STRING_AS_STRING, &iter);
-	__connman_notifier_list_registered(&iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+		CONNMAN_MANAGER_INTERFACE, "AvailableTechnologies",
+		DBUS_TYPE_STRING, __connman_notifier_list_registered, NULL);
 }
 
-static void technology_enabled(enum connman_device_type type,
+static void technology_enabled(enum connman_service_type type,
 						connman_bool_t enabled)
 {
 	GSList *list;
-	DBusMessage *signal;
-	DBusMessageIter entry, value, iter;
-	const char *key = "EnabledTechnologies";
 
 	DBG("type %d enabled %d", type, enabled);
 
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		goto done;
+	connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+		CONNMAN_MANAGER_INTERFACE, "EnabledTechnologies",
+		DBUS_TYPE_STRING, __connman_notifier_list_enabled, NULL);
 
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-					DBUS_TYPE_STRING_AS_STRING, &iter);
-	__connman_notifier_list_enabled(&iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
-
-done:
 	for (list = notifier_list; list; list = list->next) {
 		struct connman_notifier *notifier = list->data;
 
-		if (notifier->device_enabled)
-			notifier->device_enabled(type, enabled);
+		if (notifier->service_enabled)
+			notifier->service_enabled(type, enabled);
 	}
 }
 
+unsigned int __connman_notifier_count_connected(void)
+{
+	unsigned int i, count = 0;
+
+	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+		if (g_atomic_int_get(&connected[i]) > 0)
+			count++;
+	}
+
+	return count;
+}
+
+const char *__connman_notifier_get_state(void)
+{
+	unsigned int count = __connman_notifier_count_connected();
+
+	if (count > 0)
+		return "online";
+
+	return "offline";
+}
+
+static void state_changed(void)
+{
+	unsigned int count = __connman_notifier_count_connected();
+	char *state = "offline";
+	DBusMessage *signal;
+
+	if (count > 1)
+		return;
+
+	if (count > 0)
+		state = "online";
+
+	connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "State",
+						DBUS_TYPE_STRING, &state);
+
+	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "StateChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_append_args(signal, DBUS_TYPE_STRING, &state,
+							DBUS_TYPE_INVALID);
+
+	g_dbus_send_message(connection, signal);
+}
+
 static void technology_connected(enum connman_service_type type,
 						connman_bool_t connected)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value, iter;
-	const char *key = "ConnectedTechnologies";
-
 	DBG("type %d connected %d", type, connected);
 
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
+	connman_dbus_property_changed_array(CONNMAN_MANAGER_PATH,
+		CONNMAN_MANAGER_INTERFACE, "ConnectedTechnologies",
+		DBUS_TYPE_STRING, __connman_notifier_list_connected, NULL);
 
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-			DBUS_TYPE_ARRAY_AS_STRING DBUS_TYPE_STRING_AS_STRING,
-								&value);
-
-	dbus_message_iter_open_container(&value, DBUS_TYPE_ARRAY,
-					DBUS_TYPE_STRING_AS_STRING, &iter);
-	__connman_notifier_list_connected(&iter);
-	dbus_message_iter_close_container(&value, &iter);
-
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	state_changed();
 }
 
 void __connman_notifier_register(enum connman_service_type type)
@@ -255,6 +220,9 @@ void __connman_notifier_register(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -274,6 +242,9 @@ void __connman_notifier_unregister(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -293,6 +264,9 @@ void __connman_notifier_enable(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -312,6 +286,9 @@ void __connman_notifier_disable(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -331,6 +308,9 @@ void __connman_notifier_connect(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -350,6 +330,9 @@ void __connman_notifier_disconnect(enum connman_service_type type)
 
 	switch (type) {
 	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
@@ -363,29 +346,41 @@ void __connman_notifier_disconnect(enum connman_service_type type)
 		technology_connected(type, FALSE);
 }
 
+static void technology_default(enum connman_service_type type)
+{
+	const char *str;
+
+	str = __connman_service_type2string(type);
+	if (str == NULL)
+		str = "";
+
+	connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
+			CONNMAN_MANAGER_INTERFACE, "DefaultTechnology",
+						DBUS_TYPE_STRING, &str);
+}
+
+void __connman_notifier_default_changed(struct connman_service *service)
+{
+	enum connman_service_type type = connman_service_get_type(service);
+	GSList *list;
+
+	technology_default(type);
+
+	for (list = notifier_list; list; list = list->next) {
+		struct connman_notifier *notifier = list->data;
+
+		if (notifier->default_changed)
+			notifier->default_changed(service);
+	}
+}
+
 static void offlinemode_changed(dbus_bool_t enabled)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "OfflineMode";
-
 	DBG("enabled %d", enabled);
 
-	signal = dbus_message_new_signal(CONNMAN_MANAGER_PATH,
-				CONNMAN_MANAGER_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &enabled);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	connman_dbus_property_changed_basic(CONNMAN_MANAGER_PATH,
+				CONNMAN_MANAGER_INTERFACE, "OfflineMode",
+						DBUS_TYPE_BOOLEAN, &enabled);
 }
 
 void __connman_notifier_offlinemode(connman_bool_t enabled)
@@ -406,6 +401,30 @@ void __connman_notifier_offlinemode(connman_bool_t enabled)
 	}
 }
 
+connman_bool_t __connman_notifier_is_enabled(enum connman_service_type type)
+{
+	DBG("type %d", type);
+
+	switch (type) {
+	case CONNMAN_SERVICE_TYPE_UNKNOWN:
+	case CONNMAN_SERVICE_TYPE_SYSTEM:
+	case CONNMAN_SERVICE_TYPE_GPS:
+	case CONNMAN_SERVICE_TYPE_VPN:
+		return FALSE;
+	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_WIFI:
+	case CONNMAN_SERVICE_TYPE_WIMAX:
+	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
+	case CONNMAN_SERVICE_TYPE_CELLULAR:
+		break;
+	}
+
+	if (g_atomic_int_get(&enabled[type]) > 0)
+		return TRUE;
+
+	return FALSE;
+}
+
 int __connman_notifier_init(void)
 {
 	DBG("");
